<?php

namespace Drupal\graphql_vertex_ai\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * GraphQL Vertex AI settings.
 */
class GraphqlVertexAiSettingsForm extends ConfigFormBase {

  /**
   * Constructs a GraphqlVertexAiSettingsForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typedConfigManager
   *   The typed config manager.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $dateFormatter
   *   The date formatter service.
   */
  public function __construct(
    ConfigFactoryInterface $configFactory,
    TypedConfigManagerInterface $typedConfigManager,
    protected DateFormatterInterface $dateFormatter,
  ) {
    parent::__construct($configFactory, $typedConfigManager);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('date.formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'graphql_vertex_ai_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['graphql_vertex_ai.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['connection'] = [
      '#title' => $this->t('Connection'),
      '#type' => 'details',
      '#open' => TRUE,
    ];

    $form['connection']['project_id'] = [
      '#title' => $this->t('Project ID'),
      '#type' => 'textfield',
      '#description' => $this->t('Google project ID.'),
      '#config_target' => 'graphql_vertex_ai.settings:project_id',
    ];

    $form['connection']['location'] = [
      '#title' => $this->t('Location'),
      '#type' => 'textfield',
      '#description' => $this->t('Google project location.'),
      '#config_target' => 'graphql_vertex_ai.settings:location',
    ];

    $form['connection']['data_store_id'] = [
      '#title' => $this->t('Data Store ID'),
      '#type' => 'textfield',
      '#description' => $this->t('Google dataStore ID.'),
      '#config_target' => 'graphql_vertex_ai.settings:data_store_id',
    ];

    $form['connection']['serving_config_id'] = [
      '#title' => $this->t('Serving Config ID'),
      '#type' => 'textfield',
      '#description' => $this->t('Google servingConfig ID. Usually the ID of your Agent in Agent Builder.'),
      '#config_target' => 'graphql_vertex_ai.settings:serving_config_id',
    ];

    $form['connection']['key'] = [
      '#title' => 'Key',
      '#type' => 'key_select',
      '#description' => $this->t('Service account authentication_multivalue key.'),
      '#key_filters' => [
        'type' => 'authentication_multivalue',
      ],
      '#config_target' => 'graphql_vertex_ai.settings:key',
    ];

    // Identical options to the ones for block caching.
    // @see \Drupal\Core\Block\BlockBase::buildConfigurationForm()
    $period = [0, 60, 180, 300, 600, 900, 1800, 2700, 3600, 10800, 21600, 32400, 43200, 86400];
    $period = array_map([$this->dateFormatter, 'formatInterval'], array_combine($period, $period));
    $form['cache_max_age'] = [
      '#title' => $this->t('Cache max-age'),
      '#type' => 'select',
      '#description' => $this->t('The cache lifetime of Vertex AI requests. It is recommended to use a value higher than 0 to protect your site from a Distributed Denial of Service attack (DDoS).'),
      '#options' => $period,
      '#config_target' => 'graphql_vertex_ai.settings:cache_max_age',
    ];

    $form['vertex_params'] = [
      '#title' => $this->t('Request parameters'),
      '#type' => 'details',
    ];

    $form['vertex_params']['facet_limit'] = [
      '#title' => $this->t('Facet limit'),
      '#type' => 'number',
      '#description' => $this->t('The maximum number of values to be returned for each facet.'),
      '#min' => 0,
      '#max' => 300,
      '#config_target' => 'graphql_vertex_ai.settings:facet_limit',
    ];

    $form['vertex_params']['autocomplete_model'] = [
      '#title' => $this->t('Autocomplete model'),
      '#type' => 'textfield',
      '#description' => $this->t('The autocomplete model to use. Leave empty to use the default.'),
      '#config_target' => 'graphql_vertex_ai.settings:autocomplete_model',
    ];

    $form['summary_settings'] = [
      '#title' => $this->t('AI Summary Settings'),
      '#type' => 'details',
      '#open' => (bool) $this->config('graphql_vertex_ai.settings')->get('summary.allow'),
    ];

    $form['summary_settings']['allow'] = [
      '#title' => $this->t('Allow AI-generated summaries for search results'),
      '#type' => 'checkbox',
      '#config_target' => 'graphql_vertex_ai.settings:summary.allow',
    ];

    $form['summary_settings']['result_count'] = [
      '#title' => $this->t('Result count'),
      '#type' => 'number',
      '#description' => $this->t('Number of results to include in the summary.'),
      '#min' => 1,
      '#config_target' => 'graphql_vertex_ai.settings:summary.result_count',
    ];

    $form['summary_settings']['model'] = [
      '#title' => $this->t('Large language model for summarization'),
      '#type' => 'select',
      '#options' => [
        'stable' => $this->t('Stable - Generally available, fine-tuned model'),
        'preview' => $this->t('Preview - Public preview model'),
        'gemini-2.0-flash-001/answer_gen/v1' => $this->t('Gemini 2.0 Flash 1'),
        'gemini-1.5-flash-002/answer_gen/v1' => $this->t('Gemini 1.5 Flash 2'),
        'gemini-1.5-flash-001/answer_gen/v2' => $this->t('Gemini 1.5 Flash 1 V2'),
        'gemini-1.5-flash-001/answer_gen/v1' => $this->t('Gemini 1.5 Flash 1'),
      ],
      '#config_target' => 'graphql_vertex_ai.settings:summary.model',
    ];

    $form['summary_settings']['system_prompt'] = [
      '#title' => $this->t('System prompt'),
      '#type' => 'textarea',
      '#rows' => 12,
      '#description' => $this->t('If desired, provide instructions to customize the tone, style and verbosity of the summary.'),
      '#config_target' => 'graphql_vertex_ai.settings:summary.system_prompt',
    ];

    $form['summary_settings']['ignore_adversarial'] = [
      '#title' => $this->t('Skip summarization for adversarial queries'),
      '#type' => 'checkbox',
      '#config_target' => 'graphql_vertex_ai.settings:summary.ignore_adversarial',
    ];

    $form['summary_settings']['ignore_jailbreak'] = [
      '#title' => $this->t('Skip summarization for jail-breaking queries'),
      '#type' => 'checkbox',
      '#config_target' => 'graphql_vertex_ai.settings:summary.ignore_jailbreak',
    ];

    $form['summary_settings']['ignore_low_relevance'] = [
      '#title' => $this->t('Skip summarization for queries with low relevant content'),
      '#type' => 'checkbox',
      '#config_target' => 'graphql_vertex_ai.settings:summary.ignore_low_relevance',
    ];

    $form['summary_settings']['ignore_non_summary'] = [
      '#title' => $this->t('Skip summarization for queries that do not seek a summary'),
      '#type' => 'checkbox',
      '#config_target' => 'graphql_vertex_ai.settings:summary.ignore_non_summary',
    ];

    return parent::buildForm($form, $form_state);
  }

}
